/*
 *   Copyright (c) 2000 S.Noda <sakae_noda@hotmail.com>
 *
 *   Permission to use, copy, modify, and distribute this software
 * and its documentation for any purpose is hereby granted provided
 * that the above copyright notice and this permission notice appear
 * in all copies of the software and related documentation.
 *
 *                          NO WARRANTY
 *
 *   THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY WARRANTIES;
 * WITHOUT EVEN THE IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS
 * FOR A PARTICULAR PURPOSE.
 */

#include <windows.h>
#include <stdio.h>
#include "dmish32.h"

#define DMISH32_VERSION 1
#define BUFFSIZE (16 * 1024) // 16kB
#define Number(arr) (sizeof(arr) / sizeof(arr[0]))

typedef int (WINAPI * ISH)(const HWND, LPCSTR);
typedef WORD (WINAPI * ISH_GETVERSION)(VOID);
typedef BOOL (WINAPI * ISH_GETRUNNING)(VOID);
typedef BOOL (WINAPI * ISH_CONFIGDIALOG)(const HWND, LPSTR, const int);
typedef BOOL (WINAPI * ISH_CHECKARCHIVE)(LPCSTR, const int);
typedef int (WINAPI * ISH_EX)(LPCSTR, LPSTR, const DWORD);
typedef int (WINAPI * ISH_FILELIST)(LPCSTR, LPSTR, const DWORD);

typedef struct {
	HWND hwnd;
	char szFileName[FNAME_MAX32 + 1];
	DWORD dwMode;
	char szOutput[BUFFSIZE + 1];
	char *lpszOutput;
} FINDINFO;

BOOL bInit = FALSE;
HINSTANCE hDll = NULL;

static ISH fpIsh;
static ISH_GETVERSION fpIshGetVersion;
static ISH_GETRUNNING fpIshGetRunning;
static ISH_CONFIGDIALOG fpIshConfigDialog;
static ISH_CHECKARCHIVE fpIshCheckArchive;
static ISH_EX fpIshEx;
static ISH_FILELIST fpIshFileList;

BOOL WINAPI DllMain(HANDLE hModule, 
					DWORD  ul_reason_for_call, 
					LPVOID lpReserved)
{
	switch( ul_reason_for_call ) {
	case DLL_PROCESS_ATTACH:
		// DLL_Ci~bNɃ[hB
		hDll = LoadLibrary("ish32.dll");
		if (hDll == NULL) break;
		// eAPĨGg[AhX𓾂B
		fpIsh = (ISH) GetProcAddress(hDll, "Ish");
		fpIshGetVersion = (ISH_GETVERSION) GetProcAddress(hDll, "IshGetVersion");
		fpIshGetRunning = (ISH_GETRUNNING) GetProcAddress(hDll, "IshGetRunning");
		fpIshConfigDialog = (ISH_CONFIGDIALOG) GetProcAddress(hDll, "IshConfigDialog");
		fpIshCheckArchive = (ISH_CHECKARCHIVE) GetProcAddress(hDll, "IshCheckArchive");
		fpIshEx = (ISH_EX) GetProcAddress(hDll, "IshEx");
		fpIshFileList = (ISH_FILELIST) GetProcAddress(hDll, "IshFileList");
		if (fpIsh && fpIshGetVersion && fpIshGetRunning && fpIshConfigDialog &&
			fpIshCheckArchive && fpIshEx && fpIshFileList) bInit = TRUE;
		break;
	case DLL_THREAD_ATTACH:
		break;
	case DLL_THREAD_DETACH:
		break;
	case DLL_PROCESS_DETACH:
		// DLLB
		FreeLibrary(hDll);
		break;
	}
	return TRUE;
}

WORD WINAPI IshGetVersion(VOID)
{
	WORD ret = 0;
	if (bInit)
		ret = (*fpIshGetVersion)();
	return ret;
}

BOOL WINAPI IshGetRunning(VOID)
{
	BOOL ret = FALSE;
	if (bInit)
		ret = (*fpIshGetRunning)();
	return ret;
}

BOOL WINAPI IshGetBackGroundMode(VOID){return FALSE;}
BOOL WINAPI IshSetBackGroundMode(const BOOL _BackGroundMode){return FALSE;}
static BOOL s_CursorMode;
BOOL WINAPI IshGetCursorMode(VOID){return s_CursorMode;}
BOOL WINAPI IshSetCursorMode(const BOOL _CursorMode){s_CursorMode = _CursorMode; return TRUE;}
static int s_CursorInterval;
WORD WINAPI IshGetCursorInterval(VOID){return s_CursorInterval;}
BOOL WINAPI IshSetCursorInterval(const WORD _Interval){s_CursorInterval; return TRUE;}

int WINAPI Ish(const HWND _hwnd, LPCSTR _szCmdLine, LPSTR _szOutput, const DWORD _dwSize)
{
	int ret = -1;
	if (bInit)
		ret = (*fpIshEx)(_szCmdLine, _szOutput, _dwSize);
	return ret;
}

BOOL WINAPI IshCheckArchive(LPCSTR _szFileName, const int _iMode)
{
	BOOL ret = FALSE;
	if (bInit)
		ret = (*fpIshCheckArchive)(_szFileName, _iMode);
	return ret;
}

BOOL WINAPI IshConfigDialog(const HWND _hwnd, LPSTR _lpszComBuffer, const int _iMode)
{
	BOOL ret = FALSE;
	if (bInit)
		ret = (*fpIshConfigDialog)(_hwnd, _lpszComBuffer, _iMode);
	return ret;
}

HARC WINAPI IshOpenArchive(const HWND _hwnd, LPCSTR _szFileName, const DWORD _dwMode)
{
	FINDINFO *ret = NULL;
	ret = (FINDINFO*)malloc(sizeof(FINDINFO));
	if (ret) {
		ret->hwnd = _hwnd;
		strcpy(ret->szFileName, _szFileName);
		ret->dwMode = _dwMode;
	}
	return ret;
}

int WINAPI IshCloseArchive(HARC _harc)
{
	free((FINDINFO*)_harc);
	return 0;
}

int WINAPI IshFindFirst(HARC _harc, LPCSTR _szWildName, INDIVIDUALINFO *_lpSubInfo)
{
	int ret = -1;
	int n;
	FINDINFO *handle = (FINDINFO*)_harc;
	if (bInit) {
		if (!(*fpIshFileList)(handle->szFileName, handle->szOutput, BUFFSIZE)) {
			handle->szOutput[BUFFSIZE + 1] = '\0';
			handle->lpszOutput = handle->szOutput;
			for (n = 0; n < 8; n++)
				(handle->lpszOutput = strchr(handle->lpszOutput,'\n')) && handle->lpszOutput++;
			ret = IshFindNext(_harc, _lpSubInfo);
		}
	}
	return ret;
}

int WINAPI IshFindNext(HARC _harc, INDIVIDUALINFO *_lpSubInfo)
{
	int ret = -1;
	int n;
	char type[6], vol[3];
	int year, month, date, hour, min, sec;
	FINDINFO *handle = (FINDINFO*)_harc;
	if ((sscanf(handle->lpszOutput, "%s\n%d %d-%d-%d %d:%d:%d %s %d",
		&_lpSubInfo->szFileName, &_lpSubInfo->dwOriginalSize,
		&year, &month, &date, &hour, &min, &sec, &type, &vol)) > 1) {
		_lpSubInfo->dwCompressedSize = 0;
		_lpSubInfo->dwCRC = 0;
		_lpSubInfo->uFlag = 0;
		_lpSubInfo->uOSType = -1;
		_lpSubInfo->wRatio = 0;
		if (year < 80) {year = year + 100;}
		_lpSubInfo->wDate = ((year - 80) << 4 + 5) | (month << 5) | (date);
		_lpSubInfo->wTime = (hour << 5 + 6) | (min << 5) | (sec / 2);
		strcpy(_lpSubInfo->dummy1, "");
		strcpy(_lpSubInfo->szAttribute, "");
		strcpy(_lpSubInfo->szMode, "-ish-");
		for (n = 0; n < 2; n++)
			(handle->lpszOutput = strchr(handle->lpszOutput,'\n')) && handle->lpszOutput++;
		ret = 0;
	}
	return ret;
}

BOOL WINAPI IshQueryFunctionList(const int _iFunction)
{
	switch( _iFunction) {
	case ISARC:
	case ISARC_GET_VERSION:
	case ISARC_GET_CURSOR_INTERVAL:
	case ISARC_SET_CURSOR_INTERVAL:
	case ISARC_GET_BACK_GROUND_MODE:
	case ISARC_SET_BACK_GROUND_MODE:
	case ISARC_GET_CURSOR_MODE:
	case ISARC_SET_CURSOR_MODE:
	case ISARC_GET_RUNNING:
	
	case ISARC_CHECK_ARCHIVE:
	case ISARC_CONFIG_DIALOG:
	// case ISARC_GET_FILE_COUNT:
	case ISARC_QUERY_FUNCTION_LIST:
	// case ISARC_HOUT:
	// case ISARC_STRUCTOUT:
	// case ISARC_GET_ARC_FILE_INFO:
	
	case ISARC_OPEN_ARCHIVE:
	case ISARC_CLOSE_ARCHIVE:
	case ISARC_FIND_FIRST:
	case ISARC_FIND_NEXT:
	// case ISARC_EXTRACT:
	// case ISARC_ADD:
	// case ISARC_MOVE:
	// case ISARC_DELETE:
	
	// case ISARC_GET_ARC_FILE_NAME:
	// case ISARC_GET_ARC_FILE_SIZE:
	// case ISARC_GET_ARC_ORIGINAL_SIZE:
	// case ISARC_GET_ARC_COMPRESSED_SIZE:
	// case ISARC_GET_ARC_RATIO:
	// case ISARC_GET_ARC_DATE:
	// case ISARC_GET_ARC_TIME:
	// case ISARC_GET_ARC_OS_TYPE:
	// case ISARC_GET_ARC_IS_SFX_FILE:
	// case ISARC_GET_FILE_NAME:
	// case ISARC_GET_ORIGINAL_SIZE:
	// case ISARC_GET_COMPRESSED_SIZE:
	// case ISARC_GET_RATIO:
	// case ISARC_GET_DATE:
	// case ISARC_GET_TIME:
	// case ISARC_GET_CRC:
	// case ISARC_GET_ATTRIBUTE:
	// case ISARC_GET_OS_TYPE:
	// case ISARC_GET_METHOD:
	// case ISARC_GET_WRITE_TIME:
	// case ISARC_GET_CREATE_TIME:
	// case ISARC_GET_ACCESS_TIME:
		return TRUE;
	default:
		return FALSE;
	}
}
